# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-12-20 11:51
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0002_distribute'),
    ]

    operations = [
        migrations.RunSQL("ALTER TABLE tests_tenantnotidmodel DROP CONSTRAINT tests_tenantnotidmodel_pkey CASCADE;"),
        migrations.RunSQL("ALTER TABLE tests_somerelatedmodel DROP CONSTRAINT tests_somerelatedmodel_pkey CASCADE;"),

        migrations.RunSQL("SELECT create_distributed_table('tests_tenantnotidmodel', 'tenant_column');"),
        migrations.RunSQL("SELECT create_distributed_table('tests_somerelatedmodel', 'related_tenant_id');"),

        migrations.RunSQL("ALTER TABLE tests_somerelatedmodel ADD CONSTRAINT tests_somerelatedmodel_pkey PRIMARY KEY (related_tenant_id, id);"),
        migrations.RunSQL("ALTER TABLE tests_tenantnotidmodel ADD CONSTRAINT tests_tenantnotidmodel_pkey PRIMARY KEY (tenant_column);")
    ]



# -*- coding: utf-8 -*-
from django.conf.urls import include, url
from django.contrib import admin


# TODO
# Add view to verify objects in context


urlpatterns = [
    url(r"^admin/", admin.site.urls),
]



from django.db.models.sql.subqueries import DeleteQuery
from django.db.models import Q

from .utils import get_current_tenant, get_tenant_column


def wrap_get_compiler(base_get_compiler):

    def get_compiler(obj, *args, **kwargs):
        current_tenant = get_current_tenant()

        if current_tenant:
            try:
                tenant_column = get_tenant_column(obj.model)
                obj.add_q(Q(
                    **{tenant_column: current_tenant.tenant_value}
                ))
            except ValueError:
                pass

        return base_get_compiler(obj, *args, **kwargs)

    get_compiler._sign = "get_compiler django-multitenant"
    return get_compiler



# Generated by Django 2.1.1 on 2019-03-01 08:45

from django.db import migrations
import django.db.models.deletion
import django_multitenant.fields


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0005_auto_20190301_0842'),
    ]

    operations = [
        migrations.AddField(
            model_name='subtask',
            name='project',
            field=django_multitenant.fields.TenantForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='tests.Project'),
        ),
    ]



from django.db.utils import NotSupportedError
from django_multitenant.utils import set_current_tenant, unset_current_tenant

from .base import BaseTestCase
# from .models import Project, Account



class TenantModelTest(BaseTestCase):
    def test_filter_without_joins(self):
        from .models import Project
        projects = self.projects
        account = self.account_fr

        self.assertEqual(Project.objects.count(), len(projects))
        set_current_tenant(account)
        self.assertEqual(Project.objects.count(), account.projects.count())
        unset_current_tenant()


    def test_filter_without_joins_on_tenant_id_not_pk(self):
        from .models import TenantNotIdModel, SomeRelatedModel
        tenants = self.tenant_not_id

        self.assertEqual(SomeRelatedModel.objects.count(), 30)
        set_current_tenant(tenants[0])
        self.assertEqual(SomeRelatedModel.objects.count(), 10)
        unset_current_tenant()

    def test_select_tenant(self):
        from .models import Project
        self.projects
        project = Project.objects.first()

        # Selecting the account, account_id being tenant
        with self.assertNumQueries(1) as captured_queries:
            account = project.account
            self.assertTrue('"tests_account"."id" = %d' % project.account_id \
                            in captured_queries.captured_queries[0]['sql'])


    def test_select_tenant_not_id(self):
        # Test with tenant_id not id field
        from .models import SomeRelatedModel

        tenants = self.tenant_not_id
        some_related = SomeRelatedModel.objects.first()

        # Selecting the tenant, tenant_column being tenant_id in tests_tenantnodidmodel
        with self.assertNumQueries(1) as captured_queries:
            tenant = some_related.related_tenant
            self.assertTrue('"tests_tenantnotidmodel"."tenant_column" = %d' % some_related.related_tenant_id \
                            in captured_queries.captured_queries[0]['sql'])

    def test_select_tenant_foreign_key(self):
        from .models import Task
        self.tasks

        task = Task.objects.first()
        set_current_tenant(task.account)

        # Selecting task.project, account is tenant
        # To push down, account_id should be in query
        with self.assertNumQueries(1) as captured_queries:
            project = task.project
            self.assertTrue('AND "tests_project"."account_id" = %d' % task.account_id \
                            in captured_queries.captured_queries[0]['sql'])


        unset_current_tenant()

    def test_filter_select_related(self):
        from .models import Task
        task_id = self.tasks[0].pk

        # Test that the join clause has account_id
        with self.assertNumQueries(1) as captured_queries:
            task = Task.objects.filter(pk=task_id).select_related('project').first()

            self.assertEqual(task.account_id, task.project.account_id)
            self.assertTrue('INNER JOIN "tests_project" ON ("tests_task"."project_id" = "tests_project"."id" AND ("tests_task"."account_id" = ("tests_project"."account_id")))' \
                            in captured_queries.captured_queries[0]['sql'])


    def test_filter_select_related_not_id_field(self):
        from .models import SomeRelatedModel
        tenants = self.tenant_not_id
        some_related_id = SomeRelatedModel.objects.first().pk
        some_related_tenant_id = SomeRelatedModel.objects.first().related_tenant_id

        # Test that the join clause has account_id
        with self.assertNumQueries(1) as captured_queries:
            related = SomeRelatedModel.objects.filter(pk=some_related_id).select_related('related_tenant').first()

            self.assertEqual(related.related_tenant_id, some_related_tenant_id)
            self.assertTrue('ON ("tests_somerelatedmodel"."related_tenant_id" = "tests_tenantnotidmodel"."tenant_column")' \
                            in captured_queries.captured_queries[0]['sql'])


    def test_prefetch_related(self):
        from .models import Project

        project_managers = self.project_managers
        project_id = project_managers[0].project_id
        account = project_managers[0].account
        set_current_tenant(account)

        with self.assertNumQueries(2) as captured_queries:
            project = Project.objects.filter(pk=project_id).prefetch_related('managers').first()

            self.assertTrue('WHERE ("tests_manager"."account_id" = %d' % project.account_id \
                            in captured_queries.captured_queries[1]['sql'])
            self.assertTrue('AND ("tests_projectmanager"."account_id" = ("tests_manager"."account_id"))' \
                            in captured_queries.captured_queries[1]['sql'])

    def test_create_project(self):
        # Using save()
        from .models import Project
        account = self.account_fr

        project = Project()
        project.account = account
        project.name = 'test save()'

        project.save()

        self.assertEqual(Project.objects.count(), 1)

    def test_update_tenant_project(self):
        from .models import Project
        account = self.account_fr

        project = Project()
        project.account = account
        project.name = 'test save()'

        project.save()

        self.assertEqual(Project.objects.count(), 1)

        project.account = self.account_in

        with self.assertRaises(NotSupportedError):
            project.save()

        project = Project.objects.first()
        self.assertEqual(project.account, account)

    def test_save_project(self):
        from .models import Project
        account = self.account_fr

        project = Project()
        project.account = account
        project.name = 'test save()'
        project.save()

        self.assertEqual(Project.objects.count(), 1)

        project.account = account
        project.name = 'test update'
        project.save()

        project = Project.objects.first()
        self.assertEqual(project.account, account)
        self.assertEqual(project.name, 'test update')

    def test_delete_tenant_set(self):
        from .models import Project
        projects = self.projects
        account = self.account_fr

        self.assertEqual(Project.objects.count(), 30)

        set_current_tenant(account)
        with self.assertNumQueries(6) as captured_queries:
            Project.objects.all().delete()
            unset_current_tenant()

            for query in captured_queries.captured_queries:
                self.assertTrue('"account_id" = %d' % account.id in query['sql'])

        self.assertEqual(Project.objects.count(), 20)

    def test_delete_tenant_not_set(self):
        from .models import Project
        projects = self.projects

        self.assertEqual(Project.objects.count(), 30)
        Project.objects.all().delete()
        self.assertEqual(Project.objects.count(), 0)



import logging

from django.db import models
from django.db.models.sql import DeleteQuery
from django.db.models.deletion import Collector

from .deletion import related_objects
from .query import wrap_get_compiler
from .utils import (
    set_current_tenant,
    get_current_tenant,
    get_model_by_db_table,
    get_tenant_column
)


logger = logging.getLogger(__name__)



class TenantQuerySetMixin(object):
    def add_tenant_filters_with_joins(self):
        #This is adding tenant filters for all the models in the join.

        current_tenant = get_current_tenant()

        if current_tenant:
            current_tenant_id = current_tenant.tenant_value

            extra_sql = []
            extra_params = []
            current_table_name = self.model._meta.db_table
            alias_refcount = self.query.alias_refcount
            alias_map = self.query.alias_map

            for k,v in alias_refcount.items():
                if(v>0 and k!=current_table_name):
                    current_model=get_model_by_db_table(alias_map[k].table_name)

                    if issubclass(current_model, TenantModel):
                        tenant_column = get_tenant_column(current_model)
                        extra_sql.append(k + '."' + tenant_column + '" = %s')

                        extra_params.append(current_tenant_id)

            self.query.add_extra([],[],extra_sql,extra_params,[],[])

    def create(self, *args, **kwargs):
        #Below are APIs which generate SQL, this is where the tenant_id filters are injected for joins.
        current_tenant = get_current_tenant()
        if current_tenant:
            tenant_column = get_tenant_column(self.model)

            kwargs[tenant_column] = current_tenant.tenant_value

        return super(TenantQuerySetMixin, self).create(*args, **kwargs)

    def _as_sql(self, connection):
        #This API is called when there is a subquery. Injected tenant_ids for the subqueries.
        self.add_tenant_filters_with_joins()
        return super(TenantQuerySetMixin,self)._as_sql(connection)


class TenantQuerySet(TenantQuerySetMixin, models.QuerySet):
    #Modified QuerySet to add suitable filters on joins.
    pass


class TenantManagerMixin(object):
    #Below is the manager related to the above class.

    def get_queryset(self):
        #Injecting tenant_id filters in the get_queryset.
        #Injects tenant_id filter on the current model for all the non-join/join queries. 

        queryset = TenantQuerySet(self.model)
        current_tenant = get_current_tenant()
        if current_tenant:
            kwargs = { self.model.tenant_id: current_tenant.tenant_value}

            return queryset.filter(**kwargs)
        return queryset


class TenantModelMixin(object):
    #Abstract model which all the models related to tenant inherit.
    tenant_id = ''

    def __init__(self, *args, **kwargs):
        if not hasattr(DeleteQuery.get_compiler, "_sign"):
            DeleteQuery.get_compiler = wrap_get_compiler(DeleteQuery.get_compiler)
            Collector.related_objects = related_objects

        super(TenantModelMixin, self).__init__(*args, **kwargs)

    def _do_update(self, base_qs, using, pk_val, values, update_fields, forced_update):
        #adding tenant filters for save
        #Citus requires tenant_id filters for update, hence doing this below change.

        current_tenant = get_current_tenant()

        if current_tenant:
            kwargs = { self.__class__.tenant_id: current_tenant.tenant_value}
            base_qs = base_qs.filter(**kwargs)
        else:
            logger.warning('Attempting to update %s instance "%s" without a current tenant '
                           'set. This may cause issues in a partitioned environment. '
                           'Recommend calling set_current_tenant() before performing this '
                           'operation.',
                           self._meta.model.__name__, self)

        return super(TenantModelMixin,self)._do_update(base_qs, using,
                                                       pk_val, values,
                                                       update_fields,
                                                       forced_update)

    @property
    def tenant_field(self):
        return self.tenant_id

    @property
    def tenant_value(self):
        return getattr(self, self.tenant_field, None)



import logging
from django.apps import apps
from django.db import models
try:
    from threading import local
except ImportError:
    from django.utils._threading_local import local

from collections import OrderedDict

# file kept for compatibility with old version of the library

from .fields import TenantForeignKey, TenantOneToOneField
from .models import TenantManager, TenantModel, TenantQuerySet
from .thread import ThreadLocals
from .utils import (get_current_user, get_model_by_db_table,
                    get_current_tenant, set_current_tenant)



# commit in future

from django.conf import settings
from django.apps import apps


from .settings import (TENANT_COOKIE_NAME,
                       TENANT_MODEL_NAME)


class TenantMiddleware(object):
    def __call__(self, request):
        try:
            tenant_id = request.COOKIES.get(TENANT_COOKIE_NAME) or request.session.tenant_id
        except:  # add exception
            raise ImproperlyConfigured('message')

        if not tenant_id:
            return

        # find the settings TENANT_MODEL_NAME
        request.session.tenant_id = tenant_id
        tenant_model = apps.get_model(*TENANT_MODEL_NAME.split('.'))
        request.session.tenant = tenant_model.objects.get(**{tenant_model.tenant_id: tenant_id})

    def process_request(self, request):
        self.__call__(request)  # compat django 1.9



# utils.py
def get_current_tenant_from_request(request):
    '''
    Can be used in a view, if using the TenantMiddleware
    '''
    return request.session.tenant or get_current_tenant()



from .utils import get_current_tenant, get_tenant_column



def related_objects(obj, related, objs):
    filters = {
        "%s__in" % related.field.name: objs
    }
    current_tenant = get_current_tenant()

    if current_tenant:
        try:
            tenant_column = get_tenant_column(related.related_model)
            tenant_value = getattr(current_tenant, current_tenant.tenant_id, None)
            filters[tenant_column] = tenant_value
        except ValueError:
            pass

    return related.related_model._base_manager.using(obj.using).filter(**filters)



import inspect

from django.apps import apps

try:
    from threading import local
except ImportError:
    from django.utils._threading_local import local


_thread_locals = local()


def get_current_user():
    """
    Despite arguments to the contrary, it is sometimes necessary to find out who is the current
    logged in user, even if the request object is not in scope.  The best way to do this is
    by storing the user object in middleware while processing the request.
    """
    return getattr(_thread_locals, 'user', None)

def get_model_by_db_table(db_table):
    for model in apps.get_models():
        if model._meta.db_table == db_table:
            return model
    else:
        # here you can do fallback logic if no model with db_table found
        raise ValueError('No model found with db_table {}!'.format(db_table))
        # or return None

def get_current_tenant():
    """
    To get the Tenant instance for the currently logged in tenant.
    example:
        tenant = get_current_tenant()
    """
    # tenant may not be set yet, if request user is anonymous, or has no profile,
    return getattr(_thread_locals, 'tenant', None)


def get_tenant_column(model_class_or_instance):
    if inspect.isclass(model_class_or_instance):
        model_class_or_instance = model_class_or_instance()

    try:
        return model_class_or_instance.tenant_field
    except:
        raise ValueError('''%s is not an instance or a subclass of TenantModel
                         or does not inherit from TenantMixin'''
                         % model_class_or_instance.__class__.__name__)


def get_tenant_field(model_class_or_instance):
    tenant_column = get_tenant_column(model_class_or_instance)
    all_fields = model_class_or_instance._meta.fields
    try:
        return next(field for field in all_fields if field.column == tenant_column)
    except StopIteration:
        raise ValueError('No field found in {} with column name "{}"'.format(
                         model_class_or_instance, tenant_column))


def set_current_tenant(tenant):
    setattr(_thread_locals, 'tenant', tenant)


def unset_current_tenant():
    setattr(_thread_locals, 'tenant', None)



# -*- coding: utf-8 -*-
import os
import django

BASE_PATH = os.path.normpath(
    os.path.join(os.path.dirname(os.path.abspath(__file__)), os.pardir)
)

DATABASES = {
    "default": {
        'ENGINE': 'django_multitenant.backends.postgresql',
        "NAME": "postgres",
        "USER": "postgres",
        "PASSWORD": "",
        "HOST": "localhost",
        "PORT": 5600,
        "TEST": {
            "NAME": "postgres",
            "SERIALIZE": False
        }
    }
}


# DATABASES = {
#     "default": {
#         "ENGINE": "django.db.backends.postgresql_psycopg2",
#         "NAME": "citus",
#         "USER": "citus",
#         "PASSWORD": "GB7TYh-6ITjhmpOu2uClOQ",
#         "HOST": "c.fpt7dawylvzhhdmd2uitsoaoqpq.db.citusdata.com",
#         "PORT": 5432,
#         'OPTIONS': {
#             'sslmode': 'require',
#         },
#         "TEST": {
#             "NAME": "citus",
#             "SERIALIZE": False
#         }
#     }
# }

SITE_ID = 1
DEBUG = True

MIDDLEWARE_CLASSES = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
)

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django_multitenant",
    "django_multitenant.tests",
]

SECRET_KEY = "blabla"

ROOT_URLCONF = "django_multitenant.tests.urls"



from django_multitenant.utils import (set_current_tenant,
                                      get_current_tenant,
                                      unset_current_tenant)

from .base import BaseTestCase


class UtilsTest(BaseTestCase):
    def test_set_current_tenant(self):
        from .models import Project
        projects = self.projects
        account = projects[0].account

        set_current_tenant(account)
        self.assertEqual(get_current_tenant(), account)
        unset_current_tenant()

    def test_get_current_tenant(self):
        pass

    def test_get_tenant_column(self):
        pass

    def test_get_tenant_field(self):
        pass



import uuid

from django.db import models
from django.db.models.signals import pre_save
from django.dispatch import receiver


from django_multitenant.models import TenantModel
from django_multitenant.fields import TenantForeignKey


class Country(models.Model):
    name = models.CharField(max_length=255)

class Account(TenantModel):
    name = models.CharField(max_length=255)
    domain = models.CharField(max_length=255)
    subdomain = models.CharField(max_length=255)
    country = models.ForeignKey(Country, on_delete=models.CASCADE)

    # TODO change to Meta
    tenant_id = 'id'


class Manager(TenantModel):
    name = models.CharField(max_length=255)
    account = models.ForeignKey(Account, on_delete=models.CASCADE,
                                related_name='managers')

    tenant_id = 'account_id'


class Project(TenantModel):
    name = models.CharField(max_length=255)
    account = models.ForeignKey(Account, related_name='projects',
                                on_delete=models.CASCADE)
    managers = models.ManyToManyField(Manager, through='ProjectManager')
    tenant_id = 'account_id'


class ProjectManager(TenantModel):
    project = TenantForeignKey(Project, on_delete=models.CASCADE)
    manager = TenantForeignKey(Manager, on_delete=models.CASCADE)
    account = models.ForeignKey(Account, on_delete=models.CASCADE)

    tenant_id = 'account_id'



class Task(TenantModel):
    name = models.CharField(max_length=255)
    project = TenantForeignKey(Project, on_delete=models.CASCADE,
                               related_name='tasks')
    account = models.ForeignKey(Account, on_delete=models.CASCADE)

    tenant_id = 'account_id'


class SubTask(TenantModel):
    name = models.CharField(max_length=255)
    type = models.CharField(max_length=255)
    account = models.ForeignKey(Account, on_delete=models.CASCADE)
    task = TenantForeignKey(Task, on_delete=models.CASCADE)
    project = TenantForeignKey(Project, on_delete=models.CASCADE, null=True)

    tenant_id = 'account_id'

class UnscopedModel(models.Model):
    name = models.CharField(max_length=255)


class AliasedTask(TenantModel):
    project_alias = TenantForeignKey(Project, on_delete=models.CASCADE)
    account = models.ForeignKey(Account, on_delete=models.CASCADE)

    tenant_id = 'account_id'


# Models for UUID tests
class Organization(TenantModel):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=255)
    tenant_id = 'id'


class Record(TenantModel):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=255)
    organization = TenantForeignKey(Organization, on_delete=models.CASCADE)

    tenant_id = 'organization_id'



class TenantNotIdModel(TenantModel):
    tenant_column = models.IntegerField(primary_key=True, editable=False)
    name = models.CharField(max_length=255)

    tenant_id = 'tenant_column'



class SomeRelatedModel(TenantModel):
    related_tenant = models.ForeignKey(TenantNotIdModel, on_delete=models.CASCADE)
    name = models.CharField(max_length=255)

    tenant_id = 'related_tenant_id'



from django.conf import settings

APP_NAMESPACE = "DJANGO_MULTITENANT"

TENANT_COOKIE_NAME = getattr(settings, 'TENANT_COOKIE_NAME', 'tenant_id')
TENANT_MODEL_NAME = getattr(settings, 'TENANT_MODEL_NAME', None)



# Copyright (C) 2008, 2009 Michael Trier and contributors
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the GitPython project nor the names of its contributors
#   may be used to endorse or promote products derived from this software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os.path
import sys

# Version file managment scheme and graceful degredation for
# setuptools borrowed and adapted from GitPython.
try:
    from setuptools import setup, find_packages

    # Silence pyflakes
    assert setup
    assert find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages


from setuptools import setup, find_packages
setup(
    name="django-multitenant",
    version="2.0.4",
    packages=find_packages(),

    #install_requires=['gevent>=1.1.1'],
    #extras_require={
    #    'aws': ['boto>=2.40.0'],
    #    'azure': ['azure>=1.0.3'],
    #    'google': ['google-cloud-storage>=0.22.0'],
    #    'swift': ['python-swiftclient>=3.0.0',
    #              'python-keystoneclient>=3.0.0']
    #},

    author="Django-Multitenant Author",
    author_email="sai@citusdata.com",
    maintainer="Sai Srirampur",
    maintainer_email="sai@citusdata.com",
    description="Django Library to Implement Multi-tenant datbases",
    #long_description=read('README.rst'),
    classifiers=[
        'Topic :: Database',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5'
    ],
    platforms=['any'],
    license="BSD",
    keywords=("citus django multi tenant"
              "django postgres multi-tenant"),
    url="https://github.com/citusdata/django-multitenant")

    # Include the VERSION file
    #package_data={'django-multitenant': ['0.1']})

    # install
    #entry_points={'console_scripts': ['django-multitenant=django-multitenant.cmd:main']})



class ThreadLocals(object):
    """Middleware that gets various objects from the
    request object and saves them in thread local storage."""
    def process_request(self, request):
        _thread_locals.user = getattr(request, 'user', None)

        # Attempt to set tenant
        # TODO: change, user does not necessarily have a profile
        if _thread_locals.user and not _thread_locals.user.is_anonymous():
            try:
                profile = _thread_locals.user.get_profile()
                if profile:
                    _thread_locals.tenant = getattr(profile, 'tenant', None)
            except:
                raise ValueError(
                    """A User was created with no profile.  For security reasons, 
                    we cannot allow the request to be processed any further.
                    Try deleting this User and creating it again to ensure a 
                    UserProfile gets attached, or link a UserProfile 
                    to this User.""")
